<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Dataset Files</title>
</head>
<body>
    <h1>Dataset Files</h1>
    <h2>{{ dataset_name }}</h2>
    <table>
        <thead>
            <tr>
                <th>Timestamp</th>
                <th>Files</th>
            </tr>
        </thead>
        <tbody>
            {% for timestamp, files in grouped_files %}
                <tr>
                    <td>{{ timestamp }}</td>
                    <td>
                        <ul>
                            {% for file in files %}
                                <li><a href="{{ url_for('download_file', path=file) }}">Download {{ file.name }}</a></li>
                            {% endfor %}
                        </ul>
                    </td>
                </tr>
            {% endfor %}
        </tbody>
    </table>
</body>
</html>
